#Example xdecorations config file should be in ~/.config and named xdecorations.rc
#Lines beginning with '#' are comments and ignored.
#To use the default values either comment out the relevant line or remove it.
#
#All values have a command line equivalent, additionally some switches have the form -no-XXX or -XXX, these are simple on/off switches and take no argument on the commandline, but to use them in the config file you need to supply a 0 or 1,ie to either use double buffering or not you would use:
#xdecorations -usebuffer
#or
#xdecorations -no-usebuffer
#
#Command line options override values in this file.
#
#
##### Application config #####
#Default config file is ~/.config/xdecorations.rc
#An alternate config file can be loaded with the command line switch "-configfile /path/to/config/file", or disable loading of default config file with the command line switch "-noconfig" which must be the fitsr option on the command line.

#Set theme for image files.
theme Xmas
#Default Xmas

#Set main delay, this will adjust the overall speed of the program keeping the relative delays of the various animations the same.
#If the delay is to small ( <1000, depending on the system ) you may find the graphics breaking up and the program will hog too much processor time, 8000 to 10000 is a good compromise.
#defaults to 25000
delay 25000

#Write direct to the root window(0) or to a transparent window(1).
usewindow 1
#Defaults is to use transparent windows

#Use double buffering (0) no double buffer, (1) use double buffer.
#Using double buffering gives best graphical results but is slower
usebuffer 1
#The default is to use double buffering

##### Screen Lamps #####
#The lamp set to use, 0=disable lamps.
#Defaults to 1
lampset 1

#The absolute y position of the lamps on the screen.
#Defaults to 0
lampy 0

#The delay between the lamp animation.
#Defaults to 100
lampdelay 20

#Flash type for lamps.
#0=flash, 1=inverted chase, 2=chase ,3=random, 4=cycle
#Default is flash.
lampflash 1

#Delay between cycling to next flash type.
#Defaults to 30
lampcycledelay 40

##### Flying objects #####
#Show flyers, 1=show, 0=don't show.
#Defaults to 0,
flyer 1

#Number of flying objects to use <100
#If the number of flyers are > 0 then the image is chosen randomly from the available flyers
#If the number of flyers are < 0 then tthe image is chosen sequentially
#defaults to 20
maxflyers 1

#The lowest point that the flyers will fly at, good for confining them to the top screen in a multi-screen setup.
#Defaults to 400
flyermaxy 600

#The random delay between flyers appearing on screen, larger=less frequently.
#Defaults to 500
spread 500

#Set delay for flying objects, larger delay slower the movement, should be kept quite low,
#Defaults to 1
flydelay 1

#Set the number of pixels to move, larger number=faster, lower number=smoother, should be kept quite low,
#Defaults to 8
flystep 8

#Delay for animations of flying objects;
#Defaults to 8
flyanimdelay 6

##### Tree and tree decorations. #####

#Number of tree to use.
#If no trees are shown star/treelights/tinsel are also disabled
#Defaults to 0.
tree 1

#The absolute x position of the tree on the screen.
#Defaults to 0
treex 500

#The absolute y position of the tree on the screen.
#Defaults to 0
treey 350

#You can also use the terms left/centre/right with treex to position the tree
#You can also use the terms top/centre/bottom with treey to position the tree

#Show star, 1=show, 0=don't show.
#Defaults to 0.
star 1

#The delay between the 'star' animations, the star does not have to be a star, for instance in the Halloween theme it is actually the eyes of the tree.
#Defaults to 100
stardelay 10

#Show tinsel, 1=show, 0=don't show.
#Defaults to 0.
tinsel 1

#Lampset to use on the tree, 0=disable tree lights.
#Defaults to 1
treelampset 1

#The delay between the tree lamp animation. 
#Defaults to 100
treelampdelay 20

##### Figure #####
#The figure to use, 0=disable figure
#Default to 1
figure 1

#The absolute x position of the figure on the screen.
#Defaults to 100
figurex 100

#The absolute y position of the figure on the screen.
#Defaults to 100
figurey 100

#You can also use the terms left/centre/right with figurex to position the figure
#You can also use the terms top/centre/bottom with figurey to position the figure

#The delay between the figure animation. 
#Defaults to 100
figuredelay 100

##### Falling Objects #####

#The set number of the falling objects to use, 0=disbale falling objects.
#Defaults to 1
falling 1

#Delay for falling objects
#Defaults to 1
falldelay 1

#Maximum number of falling objects <5000.
#Defaults to 100
maxfalling 100

#Random delay between new falling objects appearing
#Defaults to 1000
fallingspread 2000

#Distance falling objects move in Y direction in one go
#Defaults to 1
fallingspeed 1

#Minimum distance falling objects move in th Y direction in one go ( MUST be <= fallingspeed )
#Defaults to 1
minfallspeed 1

#Falling objects are randomly set to fall at a rate between fallingspeed and minfallspeed, to make all objects fall at the same rate make both values the same

#Falling objects max drift in X direction
#Defaults to 4
drift 4

#Animation delay for falling objects
#Defaults to 8
fallinanimdelay 6

##### Wind #####
#Wind speed/direction ( +INTEGER wind>>> -INTEGER wind<<< 0 no wind )
#Defaults to 0
wind 0

#Use/don't use gusts of wind
#Defaults to 1
usegusts 1

#Duration of gusts of wind.
#Defaults to 3000
gustlen 3000

#Random delay between gusts of wind
#Defaults to 10000
gustdelay 10000

#Speed of gusts of wind
#Defaults to 40
gustspeed 60

##### Settling #####
#Settle snow etc height at bottom of desktop.
#Defaults to 100.
#Set to 0 to disable settling.
bsettleheight 100

#Settle snow etc height on windows.
#Defaults to 100.
#Set to 0 to disable settling.
wsettleheight 50

#Settle rate of snow etc.
#Defaults to 2
#the lower the number the slower snow etc settles but the better it looks.
#Gusts should be enabled as a gust of wind will clear the settled snow.
#You should set drift >1 for a more realistic effect.
#The settle rate should be lower than the height of the smallest falling object.
settlerate 2

#Clear snow etc when it reaches the maximum height.
#Defaults to 1
clearonmaxheight 1

#Offset the start point of settling snow etc from the top of the windows.
#Defaults to 0
woffsety	0

#Offset the start point of settling snow etc from the bottom of the screen.
#Defaults to 0
boffsety	0

#### EXTRAS #####
#These are not used in the config file but are command line switches ONLY.
#Don't load the default config file ( ~/.config/xdecorations.rc ).
#MUST be the firts option given.
#-noconfig

#Set new config file ( using only one instance of this makes any sense )
#This will overwite any options used or any options set in the default config.
#-configfile "/path/to/config/file"

#Write out a new config file including currently loaded config file,defaults and command line options then exit
#-writeconfig  "/path/to/config/file"

#Watch the last loaded config file ( which may be an empty file ) for changes and apply them in real time.
#this is experimental at the moment and will probably be changed in future releases.
#-watchconfig











